﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Gildor.HeatMapDemos.SL
{
    public class HeatMapGenerator
    {
        private readonly int[] _palette = new int[256];
        private readonly int _ellipseRadius;
        private readonly float _heatMapGradientStop;
        private readonly byte _intensity;

        private readonly RadialGradientBrush _ellipseBrush;

        /// <summary>
        /// Initializes a new instance of the <see cref="HeatMapGenerator"/> class.
        /// </summary>
        /// <param name="ellipseRadius">The ellipse radius.</param>
        /// <param name="heatMapGradientStop">The heat map gradient stop.</param>
        public HeatMapGenerator (
            int ellipseRadius,
            float heatMapGradientStop,
            byte intensity)
        {
            _ellipseRadius = ellipseRadius;
            _heatMapGradientStop = heatMapGradientStop;
            _intensity = intensity;
            _ellipseBrush = createBrushForEllipse ();

            // Load the palatte.
            var source = new BitmapImage (new Uri ("/Gildor.HeatMapDemos.SL;component/Palette/default.png", UriKind.Relative));
            source.CreateOptions = BitmapCreateOptions.IgnoreImageCache;
            var paletteBmp = new WriteableBitmap (source);
            _palette = paletteBmp.Pixels;
            _palette[255] = 0;
        }

        /// <summary>
        /// Generate a heat map for the specified locations.
        /// </summary>
        /// <param name="positions">The locations.</param>
        /// <returns>The heat map generated.</returns>
        public ImageSource GenerateHeatMap (IEnumerable<Point> positions, Size heatMapSize)
        {
            var container = createCanvas (heatMapSize);
            var positionCount = 0;
            foreach (var position in positions)
            {
                var ellipse = createEllipse ();
                container.Children.Add (ellipse);
                setEllipsePosition (ellipse, position);

                ++positionCount;
            }

            return positionCount != 0 ? drawHeatMap (container) : null;
        }

        /// <summary>
        /// Creates the canvas to draw ellipses in it.
        /// </summary>
        /// <returns></returns>
        private Canvas createCanvas (Size canvasSize)
        {
            return new Canvas ()
            {
                Height = canvasSize.Height,
                Width = canvasSize.Width
            };
        }

        /// <summary>
        /// Draws the heat map from the snap shot of the container canvas.
        /// </summary>
        /// <param name="container">The container.</param>
        /// <returns></returns>
        private WriteableBitmap drawHeatMap (Canvas container)
        {
            // Take the snapshot.
            var bmp = new WriteableBitmap (container, null);

            // Colorize.
            for (int i = 0; i < bmp.Pixels.Length; i++)
            {
                if (bmp.Pixels[i] != 0)
                {
                    bmp.Pixels[i] = _palette[(byte)~(((uint)bmp.Pixels[i]) >> 24)];
                }
            }
            return bmp;
        }

        /// <summary>
        /// Sets the ellipse to the specified position in canvas.
        /// </summary>
        /// <param name="ellipse">The ellipse.</param>
        /// <param name="position">The position.</param>
        private static void setEllipsePosition (Ellipse ellipse, Point position)
        {
            Canvas.SetLeft (ellipse, position.X - ellipse.Width / 2);
            Canvas.SetTop (ellipse, position.Y - ellipse.Height / 2);
        }

        /// <summary>
        /// Creates the ellipse with the pre-defined properties.
        /// </summary>
        /// <returns></returns>
        private Ellipse createEllipse ()
        {
            return new Ellipse ()
            {
                HorizontalAlignment = HorizontalAlignment.Left,
                VerticalAlignment = VerticalAlignment.Top,
                Height = 2 * _ellipseRadius,
                Width = 2 * _ellipseRadius,
                Fill = _ellipseBrush
            };
        }

        /// <summary>
        /// Creates the brush for ellipse.
        /// </summary>
        /// <returns></returns>
        private RadialGradientBrush createBrushForEllipse ()
        {
            var brush = new RadialGradientBrush ();

            brush.GradientStops.Add (new GradientStop ()
            {
                Color = Color.FromArgb (_intensity, 0x00, 0x00, 0x00),
                Offset = 0
            });

            brush.GradientStops.Add (new GradientStop ()
            {
                Color = Color.FromArgb (_intensity, 0x00, 0x00, 0x00),
                Offset = _heatMapGradientStop
            });

            brush.GradientStops.Add (new GradientStop ()
            {
                Color = Color.FromArgb (0x00, 0x00, 0x00, 0x00),
                Offset = 1
            });
            return brush;
        }
    }
}
