﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Windows.Forms;

namespace Gildor.HeatMapDemos.WinForm
{
    public partial class Main : Form
    {
        #region [Fields and Properties]

        private static readonly Random _rand = new Random ();

        /// <summary>
        /// The points on the map.
        /// </summary>
        private readonly List<Point> _heatPoints = new List<Point> ();

        private float _brushStop = 0.5F;
        private int _heatPointRadius = 7;
        private byte _intensity = 0x5f;

        #endregion

        #region [Constructors]

        public Main ()
        {
            InitializeComponent ();
        }

        #endregion

        #region [Draw Mask Stuff]

        private void drawMask ()
        {
            try
            {
                // Create new memory bitmap the same size as the picture box.
                // Set its format to 32bit argb to support transparency.
                Bitmap bmp = new Bitmap (pictureBox1.Width, pictureBox1.Height,
                    PixelFormat.Format32bppArgb);

                // Draws the heat points on the bitmap.
                doDrawMask (bmp, _heatPoints);

                // Put the bitmap to the PictureBox.
                pictureBox1.Image = bmp;
            }
            catch (Exception ex)
            {
                MessageBox.Show (ex.ToString (), "绘制灰度图",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void doDrawMask (Bitmap bmp, List<Point> heatPoints)
        {
            // Create new graphics surface from the bitmap.
            Graphics surface = Graphics.FromImage (bmp);

            ColorBlend colors = getColorBlend ();

            // Draw mask heat points on the surface.
            foreach (Point heatPoint in heatPoints)
            {
                drawHeatPoint (surface, heatPoint, colors, _heatPointRadius);
            }
        }

        private void drawHeatPoint (Graphics surface, Point heatPoint,
            ColorBlend colors, int radius)
        {
            // Create the ellipse path.
            var ellipsePath = new GraphicsPath ();
            ellipsePath.AddEllipse (heatPoint.X - radius, heatPoint.Y - radius,
                radius * 2, radius * 2);

            // Create the brush.
            PathGradientBrush brush = new PathGradientBrush (ellipsePath);
            ColorBlend gradientSpecifications = colors;
            brush.InterpolationColors = gradientSpecifications;

            // Use the brush to fill the ellipse.
            surface.FillEllipse (brush, heatPoint.X - radius,
                heatPoint.Y - radius, radius * 2, radius * 2);
        }

        private ColorBlend getColorBlend ()
        {
            ColorBlend colors = new ColorBlend (3);

            // Set brush stops.
            colors.Positions = new float[3] { 0, _brushStop, 1 };

            // The intensity value adjusts alpha of gradient colors.
            colors.Colors = new Color[3]
            {
                Color.FromArgb(0, Color.White),
                // The following colors can be any color - Only the alpha  value is used.
                Color.FromArgb(_intensity, Color.Black),
                Color.FromArgb(_intensity, Color.Black)
            };
            return colors;
        }

        #endregion

        #region [Colorize Stuff]

        private static Bitmap doColorize (Bitmap originalMask, int[] palette)
        {
            // Create an empty bitmap for output.
            Bitmap output = new Bitmap (originalMask.Width, originalMask.Height,
                PixelFormat.Format32bppArgb);

            for (int y = 0; y < originalMask.Height; y++)
            {
                for (int x = 0; x < originalMask.Width; x++)
                {
                    // Calucate the pixel of output image according to the original pixel and palette.
                    output.SetPixel (x, y, Color.FromArgb (
                        palette[(byte)~(((uint)(originalMask.GetPixel (x, y).ToArgb ())) >> 24)]));
                }
            }
            return output;
        }

        private void colorize ()
        {
            // Cancel the operation if the palette is not specified.
            if (txtPaletteFileName.Text.Length == 0)
            {
                return;
            }

            try
            {
                // Loads the palette.
                int[] palette = loadPalette ();

                // Colorize the original bitmap.
                var coloredMap = doColorize ((Bitmap)pictureBox1.Image, palette);

                pictureBox2.Image = coloredMap;
                pictureBox2.Refresh ();
            }
            catch (Exception ex)
            {
                MessageBox.Show (ex.ToString (), "色彩化",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Loads the palette from the file.
        /// </summary>
        /// <returns></returns>
        private int[] loadPalette ()
        {
            int[] palette = new int[256];
            Bitmap paletteImage = (Bitmap)Bitmap.FromFile (txtPaletteFileName.Text);
            for (int i = 0; i < palette.Length - 1; i++)
            {
                palette[i] = paletteImage.GetPixel (i, 0).ToArgb ();
            }
            // Set the last color to 0x00000000 to make sure areas 
            // with no heat point remain original.
            palette[palette.Length - 1] = 0;
            return palette;
        }

        #endregion

        #region [Private Event Handlers]

        private void btnGenerateIntensityMask_Click (object sender, EventArgs e)
        {
            for (int i = 0; i < 100; i++)
            {
                // Pick random locations and intensity
                int iX = _rand.Next (0, pictureBox1.Width);
                int iY = _rand.Next (0, pictureBox1.Height);

                _heatPoints.Add (new Point (iX, iY));
            }

            drawMask ();
        }

        private void btnColorize_Click (object sender, EventArgs e)
        {
            if (txtPaletteFileName.Text.Length == 0)
            {
                MessageBox.Show ("请先选择调色板文件。", "色彩化",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                btnOpenPaletteFile.Focus ();
                return;
            }
            colorize ();
        }

        private void btnOpenPaletteFile_Click (object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog () == DialogResult.OK)
            {
                txtPaletteFileName.Text = openFileDialog1.FileName;
            }
        }

        private void btnSelectBackground_Click (object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog () == DialogResult.OK)
            {
                pictureBox1.BackgroundImage = pictureBox2.BackgroundImage =
                    Image.FromFile (openFileDialog1.FileName);
            }
        }

        private void pictureBox1_MouseDown (object sender, MouseEventArgs e)
        {
            // When left clicked, add a heat point; otherwise, clear heat points.
            if (e.Button == MouseButtons.Left)
            {
                _heatPoints.Add (new Point (e.X, e.Y));
            }
            else
            {
                _heatPoints.Clear ();
            }

            drawMask ();
            //colorize ();
        }

        private void tbRadius_Scroll (object sender, EventArgs e)
        {
            _heatPointRadius = tbRadius.Value;
            lbRadius.Text = tbRadius.Value.ToString ();
            drawMask ();
            colorize ();
        }

        private void tbBrushStop_Scroll (object sender, EventArgs e)
        {
            _brushStop = tbBrushStop.Value / 100F;
            lbBrushStop.Text = _brushStop.ToString ();
            drawMask ();
            colorize ();
        }

        private void tbIntensity_Scroll (object sender, EventArgs e)
        {
            _intensity = (byte)tbIntensity.Value;
            lbIntensity.Text = _intensity.ToString ();
            drawMask ();
            colorize ();
        }

        #endregion
    }
}
